SystemOrganization addCategory: #'Pier-Squeak-Persistency'!

PRHistoryPersistency subclass: #PRImagePersistency
	instanceVariableNames: 'duration process maxImages namePrefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Squeak-Persistency'!

!PRImagePersistency commentStamp: 'lr 6/22/2008 17:53' prior: 0!
I am a persistency strategy regularly saving the whole image. Do not use me when developing, as this might corrupt the changes file.!

!PRImagePersistency methodsFor: 'actions' stamp: 'lr 6/22/2008 18:52'!
cleanup
	| images |
	maxImages isNil ifTrue: [ ^ self ].
	images := FileDirectory default fileNames select: [ :each | 
		(each endsWith: FileDirectory imageSuffix) 
			and: [ each beginsWith: namePrefix , ' ' ] ].
	images size <= maxImages ifTrue: [ ^ self ].
	images := (images asSortedCollection allButLast: maxImages)
		collect: [ :each | each copyUpToLast: $. ].
	images do: [ :each |
		FileDirectory default 
			deleteFileNamed: (each , '.' , FileDirectory imageSuffix)
			ifAbsent: [ ].
		FileDirectory default 
			deleteFileNamed: (each , '.' , FileDirectory changeSuffix)
			ifAbsent: [ ] ]! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 9/29/2006 08:39'!
duration
	^ duration! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 18:00'!
duration: aDuration
	"The minimum interval between two snapshots."

	duration := aDuration! !

!PRImagePersistency methodsFor: 'actions' stamp: 'lr 11/20/2007 16:00'!
execute: aCommand
	self mutex critical: [
		self apply: aCommand.
		aCommand isLogged ifTrue: [
			self add: aCommand context.
			self log: aCommand context ] ].
	self start! !

!PRImagePersistency methodsFor: 'initialization' stamp: 'lr 6/22/2008 18:43'!
initialize
	super initialize.
	duration := 1 hour.
	maxImages := 12.
	namePrefix := 'pier'! !

!PRImagePersistency methodsFor: 'testing' stamp: 'lr 1/17/2007 11:44'!
isRunning
	^ process notNil and: [ process isTerminated not ]! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 1/17/2007 10:37'!
log: aContext
	| stream |
	stream := FileDirectory default
		fileNamed: 'transactions.txt'.
	[ self log: aContext on: stream setToEnd ]
		ensure: [ stream close ]! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 2/12/2007 19:07'!
log: aContext on: aStream
	aStream nextPutAll: ' == '; nextPutAll: aContext command class label; nextPutAll: ' == '; cr.
	aStream nextPutAll: ' -- context -- '; cr.
	aContext description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext
				readUsing: each)); cr ].
	aStream nextPutAll: ' -- command -- '; cr.
	aContext command description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext command
				readUsing: each)); cr ].
	aStream cr; cr! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 17:54'!
maxImages
	^ maxImages! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 17:55'!
maxImages: anInteger
	"The maximum number of backup copies to keep."

	maxImages := anInteger! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 18:06'!
namePrefix
	^ namePrefix! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 18:06'!
namePrefix: aString
	"Defines the prefix for image-names."

	namePrefix := aString! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 6/22/2008 18:44'!
process
	(Delay forDuration: self duration)
		wait.
	process := nil.
	self snapshot; cleanup! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 6/22/2008 18:44'!
saveImageAs: aString
	| image |
	image := SmalltalkImage current.
	image closeSourceFiles.
	FileDirectory default 
		copyFileWithoutOverwriteConfirmationNamed: SmalltalkImage current changesName 
		toFileNamed: (FileDirectory default pathName , FileDirectory slash, aString , '.' , FileDirectory changeSuffix).
	Smalltalk setMacFileInfoOn: (FileDirectory default pathName , FileDirectory slash , aString , '.' , FileDirectory changeSuffix).
	image changeImageNameTo: (FileDirectory default pathName , FileDirectory slash , aString , '.' , FileDirectory imageSuffix).
	image openSourceFiles; saveImageSegments; snapshot: true andQuit: false! !

!PRImagePersistency methodsFor: 'initialization' stamp: 'lr 6/22/2008 18:00'!
setKernel: aKernel
	super setKernel: aKernel.
	self stop! !

!PRImagePersistency methodsFor: 'actions' stamp: 'lr 6/22/2008 18:50'!
snapshot
	"Save the whole image."

	self saveImageAs: self namePrefix , ' ' , DateAndTime now asSortableString! !

!PRImagePersistency methodsFor: 'process' stamp: 'lr 6/22/2008 18:07'!
start
	self isRunning
		ifTrue: [ ^ self ].
	process := [ self process ]
		forkAt: 30 
		named: 'pier'! !

!PRImagePersistency methodsFor: 'process' stamp: 'lr 2/12/2007 19:07'!
stop
	self isRunning
		ifTrue: [ process terminate ].
	process := nil! !

PRPersistencyTest subclass: #PRImagePersistencyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Squeak-Persistency'!

!PRImagePersistencyTest class methodsFor: 'testing' stamp: 'lr 3/30/2007 22:22'!
isAbstract
	^ false! !

!PRImagePersistencyTest methodsFor: 'private' stamp: 'lr 3/30/2007 22:22'!
actualClass
	^ PRImagePersistency! !
